package filtrexss;

import filtrexss.HTMLEntityCodec.MapElement;
import java.util.Iterator;
import java.util.Vector;

public class HTMLEntityCodec extends Codec
{
        private static final char REPLACEMENT_CHAR = '\ufffd';
        private static final String REPLACEMENT_HEX = "fffd";
        private static final String REPLACEMENT_STR = "" + REPLACEMENT_CHAR;
        private static final Vector <MapElement> characterToEntityMap = mkCharacterToEntityMap1();

      //  private static final Trie<Character> entityToCharacterTrie = mkEntityToCharacterTrie();

    /**
     *
     */
    public HTMLEntityCodec() {
        }

        /**
         * {@inheritDoc}
         *
     * Encodes a Character for safe use in an HTML entity field.
     * @param immune
     */
        public String encodeCharacter( char[] immune, Character c ) {

                // check for immune characters: comparer les caractère a une listre blache de caractères
                if ( containsCharacter(c, immune ) ) {
                        return ""+c;
                }

                // check for alphanumeric characters: 
                String hex = Codec.getHexForNonAlphanumeric(c);
               // si le type du caractère est pas alphanumerique on lui ajout un espace au debut
                if ( hex == null ) {
                        return ""+c;
                }

                // check for illegal characters: si le caractère fait partie d'un ensemble de caractère interdits on remplace par une autre valeur
                if ( ( c <= 0x1f && c != '\t' && c != '\n' && c != '\r' ) || ( c >= 0x7f && c <= 0x9f ) )
                {
                        hex = REPLACEMENT_HEX;  // Let's entity encode this instead of returning it
                        c = REPLACEMENT_CHAR;
                }

                // check if there's a defined entity: si le caractère se trouve dans la Map, le remplacer par son correspondant
                Iterator it=characterToEntityMap.iterator();
                while(it.hasNext()){
                    MapElement e=(MapElement)it.next();
                    if((e.i).equals(c));
                    String entityName = (String) e.s;
                    if (entityName != null) {
                        return "&" + entityName + ";";
                    }
                }

                // return the hex entity as suggested in the spec: retourne la valeur exa a laquelle on ajoute des caracteres
                return "&#x" + hex + ";";
        }

        /**
         * {@inheritDoc}
         *
         * Returns the decoded version of the character starting at index, or
         * null if no decoding is possible.
         *
         * Formats all are legal both with and without semi-colon, upper/lower case:
         *   &#dddd;
         *   &#xhhhh;
         *   &name;
         */
    /*    public Character decodeCharacter( PushbackString input ) {
                input.mark();
                Character first = input.next();
                if ( first == null ) {
                        input.reset();
                        return null;
                }

                // if this is not an encoded character, return null
                if (first != '&' ) {
                        input.reset();
                        return null;
                }

                // test for numeric encodings
                Character second = input.next();
                if ( second == null ) {
                        input.reset();
                        return null;
                }

                if (second == '#' ) {
                        // handle numbers
                        Character c = getNumericEntity( input );
                        if ( c != null ) return c;
                } else if ( Character.isLetter( second.charValue() ) ) {
                        // handle entities
                        input.pushback( second );
                        Character c = getNamedEntity( input );
                        if ( c != null ) return c;
                }
                input.reset();
                return null;
        }*/

        /**
         * getNumericEntry checks input to see if it is a numeric entity
         *
         * @param input
         *                      The input to test for being a numeric entity
         *
         * @return
         *                      null if input is null, the character of input after decoding
         */
   /*     private Character getNumericEntity( PushbackString input ) {
                Character first = input.peek();
                if ( first == null ) return null;

                if (first == 'x' || first == 'X' ) {
                        input.next();
                        return parseHex( input );
                }
                return parseNumber( input );
        }
*/
        /**
         * Parse a decimal number, such as those from JavaScript's String.fromCharCode(value)
         *
         * @param input
         *                      decimal encoded string, such as 65
         * @return
         *                      character representation of this decimal value, e.g. A
         * @throws NumberFormatException
         */
   /*     private Character parseNumber( PushbackString input ) {
                StringBuilder sb = new StringBuilder();
                while( input.hasNext() ) {
                        Character c = input.peek();

                        // if character is a digit then add it on and keep going
                        if ( Character.isDigit( c.charValue() ) ) {
                                sb.append( c );
                                input.next();

                        // if character is a semi-colon, eat it and quit
                        } else if (c == ';' ) {
                                input.next();
                                break;

                        // otherwise just quit
                        } else {
                                break;
                        }
                }
                try {
                        int i = Integer.parseInt(sb.toString());
            if (Character.isValidCodePoint(i)) {
                return (char) i;
            }
                } catch( NumberFormatException e ) {
                        // throw an exception for malformed entity?
                }
                        return null;
                }
*/
        /**
         * Parse a hex encoded entity
         *
         * @param input
         *                      Hex encoded input (such as 437ae;)
         * @return
         *                      A single character from the string
         * @throws NumberFormatException
         */
    /*    private Character parseHex( PushbackString input ) {
                StringBuilder sb = new StringBuilder();
                while( input.hasNext() ) {
                        Character c = input.peek();

                        // if character is a hex digit then add it on and keep going
                        if ( "0123456789ABCDEFabcdef".indexOf(c) != -1 ) {
                                sb.append( c );
                                input.next();

                        // if character is a semi-colon, eat it and quit
                        } else if (c == ';' ) {
                                input.next();
                                break;

                        // otherwise just quit
                        } else {
                                break;
                        }
                }
                try {
                        int i = Integer.parseInt(sb.toString(), 16);
            if (Character.isValidCodePoint(i)) {
                return (char) i;
            }
                } catch( NumberFormatException e ) {
                        // throw an exception for malformed entity?
                }
                        return null;
                }
*/
        /**
         *
         * Returns the decoded version of the character starting at index, or
         * null if no decoding is possible.
         *
         * Formats all are legal both with and without semi-colon, upper/lower case:
         *   &aa;
         *   &aaa;
         *   &aaaa;
         *   &aaaaa;
         *   &aaaaaa;
         *   &aaaaaaa;
         *
         * @param input
         *              A string containing a named entity like &quot;
         * @return
         *              Returns the decoded version of the character starting at index, or null if no decoding is possible.
         */
   /*     private Character getNamedEntity( PushbackString input ) {
                StringBuilder possible = new StringBuilder();
                Map.Entry<CharSequence,Character> entry;
                int len;

                // kludge around PushbackString....
                len = Math.min(input.remainder().length(), entityToCharacterTrie.getMaxKeyLength());
                for(int i=0;i<len;i++)
                        possible.append(Character.toLowerCase(input.next()));

                // look up the longest match
                entry = entityToCharacterTrie.getLongestMatch(possible);
                if(entry == null)
                        return null;    // no match, caller will reset input

                // fixup input
                input.reset();
                input.next();   // read &
                len = entry.getKey().length();  // what matched's length
                for(int i=0;i<len;i++)
                        input.next();

                // check for a trailing semicolen
                if(input.peek(';'))
                        input.next();

                return entry.getValue();
        }*/

        /**
         * Build a unmodifiable Map from entity Character to Name.
         * @return Unmodifiable map.
         */
        public static class MapElement{
            Character i;
            String s;
            public MapElement(Character c,String st){
                i=c;
                s=st;

            }

        }
        
         private static synchronized Vector <MapElement> mkCharacterToEntityMap1()
        {
             Vector map1= new Vector<MapElement>();
                //Map<Character, String> map = new HashMap<Character,String>(252);

                map1.addElement(new MapElement((char)34,       "quot"));        /* quotation mark "*/
                map1.addElement(new MapElement((char)38,       "amp"));         /* ampersand */
                map1.addElement(new MapElement((char)60,       "lt"));          /* less-than sign <*/
                map1.addElement(new MapElement((char)62,       "gt"));          /* greater-than sign >*/
                map1.addElement(new MapElement((char)160,      "nbsp"));        /* no-break space */
                map1.addElement(new MapElement((char)161,      "iexcl"));       /* inverted exclamation mark */
                map1.addElement(new MapElement((char)162,      "cent"));        /* cent sign */
                map1.addElement(new MapElement((char)163,      "pound"));       /* pound sign */
                map1.addElement(new MapElement((char)164,      "curren"));      /* currency sign */
                map1.addElement(new MapElement((char)165,      "yen"));         /* yen sign */
                map1.addElement(new MapElement((char)166,      "brvbar"));      /* broken bar */
                map1.addElement(new MapElement((char)167,      "sect"));        /* section sign */
                map1.addElement(new MapElement((char)168,      "uml"));         /* diaeresis */
                map1.addElement(new MapElement((char)169,      "copy"));        /* copyright sign */
                map1.addElement(new MapElement((char)170,      "ordf"));        /* feminine ordinal indicator */
                map1.addElement(new MapElement((char)171,      "laquo"));       /* left-pointing double angle quotation mark */
                map1.addElement(new MapElement((char)172,      "not"));         /* not sign */
                map1.addElement(new MapElement((char)173,      "shy"));         /* soft hyphen */
                map1.addElement(new MapElement((char)174,      "reg"));         /* registered sign */
                map1.addElement(new MapElement((char)175,      "macr"));        /* macron */
                map1.addElement(new MapElement((char)176,      "deg"));         /* degree sign */
                map1.addElement(new MapElement((char)177,      "plusmn"));      /* plus-minus sign */
                map1.addElement(new MapElement((char)178,      "sup2"));        /* superscript two */
                map1.addElement(new MapElement((char)179,      "sup3"));        /* superscript three */
                map1.addElement(new MapElement((char)180,      "acute"));       /* acute accent */
                map1.addElement(new MapElement((char)181,      "micro"));       /* micro sign */
                map1.addElement(new MapElement((char)182,      "para"));        /* pilcrow sign */
                map1.addElement(new MapElement((char)183,      "middot"));      /* middle dot */
                map1.addElement(new MapElement((char)184,      "cedil"));       /* cedilla */
                map1.addElement(new MapElement((char)185,      "sup1"));        /* superscript one */
                map1.addElement(new MapElement((char)186,      "ordm"));        /* masculine ordinal indicator */
                map1.addElement(new MapElement((char)187,      "raquo"));       /* right-pointing double angle quotation mark */
                map1.addElement(new MapElement((char)188,      "frac14"));      /* vulgar fraction one quarter */
                map1.addElement(new MapElement((char)189,      "frac12"));      /* vulgar fraction one half */
                map1.addElement(new MapElement((char)190,      "frac34"));      /* vulgar fraction three quarters */
                map1.addElement(new MapElement((char)191,      "iquest"));      /* inverted question mark */
                map1.addElement(new MapElement((char)192,      "Agrave"));      /* Latin capital letter a with grave */
                map1.addElement(new MapElement((char)193,      "Aacute"));      /* Latin capital letter a with acute */
                map1.addElement(new MapElement((char)194,      "Acirc"));       /* Latin capital letter a with circumflex */
                map1.addElement(new MapElement((char)195,      "Atilde"));      /* Latin capital letter a with tilde */
                map1.addElement(new MapElement((char)196,      "Auml"));        /* Latin capital letter a with diaeresis */
                map1.addElement(new MapElement((char)197,      "Aring"));       /* Latin capital letter a with ring above */
                map1.addElement(new MapElement((char)198,      "AElig"));       /* Latin capital letter ae */
                map1.addElement(new MapElement((char)199,      "Ccedil"));      /* Latin capital letter c with cedilla */
                map1.addElement(new MapElement((char)200,      "Egrave"));      /* Latin capital letter e with grave */
                map1.addElement(new MapElement((char)201,      "Eacute"));      /* Latin capital letter e with acute */
                map1.addElement(new MapElement((char)202,      "Ecirc"));       /* Latin capital letter e with circumflex */
                map1.addElement(new MapElement((char)203,      "Euml"));        /* Latin capital letter e with diaeresis */
                map1.addElement(new MapElement((char)204,      "Igrave"));      /* Latin capital letter i with grave */
                map1.addElement(new MapElement((char)205,      "Iacute"));      /* Latin capital letter i with acute */
                map1.addElement(new MapElement((char)206,      "Icirc"));       /* Latin capital letter i with circumflex */
                map1.addElement(new MapElement((char)207,      "Iuml"));        /* Latin capital letter i with diaeresis */
                map1.addElement(new MapElement((char)208,      "ETH"));         /* Latin capital letter eth */
                map1.addElement(new MapElement((char)209,      "Ntilde"));      /* Latin capital letter n with tilde */
                map1.addElement(new MapElement((char)210,      "Ograve"));      /* Latin capital letter o with grave */
                map1.addElement(new MapElement((char)211,      "Oacute"));      /* Latin capital letter o with acute */
                map1.addElement(new MapElement((char)212,      "Ocirc"));       /* Latin capital letter o with circumflex */
                map1.addElement(new MapElement((char)213,      "Otilde"));      /* Latin capital letter o with tilde */
                map1.addElement(new MapElement((char)214,      "Ouml"));        /* Latin capital letter o with diaeresis */
                map1.addElement(new MapElement((char)215,      "times"));       /* multiplication sign */
                map1.addElement(new MapElement((char)216,      "Oslash"));      /* Latin capital letter o with stroke */
                map1.addElement(new MapElement((char)217,      "Ugrave"));      /* Latin capital letter u with grave */
                map1.addElement(new MapElement((char)218,      "Uacute"));      /* Latin capital letter u with acute */
                map1.addElement(new MapElement((char)219,      "Ucirc"));       /* Latin capital letter u with circumflex */
                map1.addElement(new MapElement((char)220,      "Uuml"));        /* Latin capital letter u with diaeresis */
                map1.addElement(new MapElement((char)221,      "Yacute"));      /* Latin capital letter y with acute */
                map1.addElement(new MapElement((char)222,      "THORN"));       /* Latin capital letter thorn */
                map1.addElement(new MapElement((char)223,      "szlig"));       /* Latin small letter sharp sXCOMMAX German Eszett */
                map1.addElement(new MapElement((char)224,      "agrave"));      /* Latin small letter a with grave */
                map1.addElement(new MapElement((char)225,      "aacute"));      /* Latin small letter a with acute */
                map1.addElement(new MapElement((char)226,      "acirc"));       /* Latin small letter a with circumflex */
                map1.addElement(new MapElement((char)227,      "atilde"));      /* Latin small letter a with tilde */
                map1.addElement(new MapElement((char)228,      "auml"));        /* Latin small letter a with diaeresis */
                map1.addElement(new MapElement((char)229,      "aring"));       /* Latin small letter a with ring above */
                map1.addElement(new MapElement((char)230,      "aelig"));       /* Latin lowercase ligature ae */
                map1.addElement(new MapElement((char)231,      "ccedil"));      /* Latin small letter c with cedilla */
                map1.addElement(new MapElement((char)232,      "egrave"));      /* Latin small letter e with grave */
                map1.addElement(new MapElement((char)233,      "eacute"));      /* Latin small letter e with acute */
                map1.addElement(new MapElement((char)234,      "ecirc"));       /* Latin small letter e with circumflex */
                map1.addElement(new MapElement((char)235,      "euml"));        /* Latin small letter e with diaeresis */
                map1.addElement(new MapElement((char)236,      "igrave"));      /* Latin small letter i with grave */
                map1.addElement(new MapElement((char)237,      "iacute"));      /* Latin small letter i with acute */
                map1.addElement(new MapElement((char)238,      "icirc"));       /* Latin small letter i with circumflex */
                map1.addElement(new MapElement((char)239,      "iuml"));        /* Latin small letter i with diaeresis */
                map1.addElement(new MapElement((char)240,      "eth"));         /* Latin small letter eth */
                map1.addElement(new MapElement((char)241,      "ntilde"));      /* Latin small letter n with tilde */
                map1.addElement(new MapElement((char)242,      "ograve"));      /* Latin small letter o with grave */
                map1.addElement(new MapElement((char)243,      "oacute"));      /* Latin small letter o with acute */
                map1.addElement(new MapElement((char)244,      "ocirc"));       /* Latin small letter o with circumflex */
                map1.addElement(new MapElement((char)245,      "otilde"));      /* Latin small letter o with tilde */
                map1.addElement(new MapElement((char)246,      "ouml"));        /* Latin small letter o with diaeresis */
                map1.addElement(new MapElement((char)247,      "divide"));      /* division sign */
                map1.addElement(new MapElement((char)248,      "oslash"));      /* Latin small letter o with stroke */
                map1.addElement(new MapElement((char)249,      "ugrave"));      /* Latin small letter u with grave */
                map1.addElement(new MapElement((char)250,      "uacute"));      /* Latin small letter u with acute */
                map1.addElement(new MapElement((char)251,      "ucirc"));       /* Latin small letter u with circumflex */
                map1.addElement(new MapElement((char)252,      "uuml"));        /* Latin small letter u with diaeresis */
                map1.addElement(new MapElement((char)253,      "yacute"));      /* Latin small letter y with acute */
                map1.addElement(new MapElement((char)254,      "thorn"));       /* Latin small letter thorn */
                map1.addElement(new MapElement((char)255,      "yuml"));        /* Latin small letter y with diaeresis */
                map1.addElement(new MapElement((char)338,      "OElig"));       /* Latin capital ligature oe */
                map1.addElement(new MapElement((char)339,      "oelig"));       /* Latin small ligature oe */
                map1.addElement(new MapElement((char)352,      "Scaron"));      /* Latin capital letter s with caron */
                map1.addElement(new MapElement((char)353,      "scaron"));      /* Latin small letter s with caron */
                map1.addElement(new MapElement((char)376,      "Yuml"));        /* Latin capital letter y with diaeresis */
                map1.addElement(new MapElement((char)402,      "fnof"));        /* Latin small letter f with hook */
                map1.addElement(new MapElement((char)710,      "circ"));        /* modifier letter circumflex accent */
                map1.addElement(new MapElement((char)732,      "tilde"));       /* small tilde */
                map1.addElement(new MapElement((char)913,      "Alpha"));       /* Greek capital letter alpha */
                map1.addElement(new MapElement((char)914,      "Beta"));        /* Greek capital letter beta */
                map1.addElement(new MapElement((char)915,      "Gamma"));       /* Greek capital letter gamma */
                map1.addElement(new MapElement((char)916,      "Delta"));       /* Greek capital letter delta */
                map1.addElement(new MapElement((char)917,      "Epsilon"));     /* Greek capital letter epsilon */
                map1.addElement(new MapElement((char)918,      "Zeta"));        /* Greek capital letter zeta */
                map1.addElement(new MapElement((char)919,      "Eta"));         /* Greek capital letter eta */
                map1.addElement(new MapElement((char)920,      "Theta"));       /* Greek capital letter theta */
                map1.addElement(new MapElement((char)921,      "Iota"));        /* Greek capital letter iota */
                map1.addElement(new MapElement((char)922,      "Kappa"));       /* Greek capital letter kappa */
                map1.addElement(new MapElement((char)923,      "Lambda"));      /* Greek capital letter lambda */
                map1.addElement(new MapElement((char)924,      "Mu"));          /* Greek capital letter mu */
                map1.addElement(new MapElement((char)925,      "Nu"));          /* Greek capital letter nu */
                map1.addElement(new MapElement((char)926,      "Xi"));          /* Greek capital letter xi */
                map1.addElement(new MapElement((char)927,      "Omicron"));     /* Greek capital letter omicron */
                map1.addElement(new MapElement((char)928,      "Pi"));          /* Greek capital letter pi */
                map1.addElement(new MapElement((char)929,      "Rho"));         /* Greek capital letter rho */
                map1.addElement(new MapElement((char)931,      "Sigma"));       /* Greek capital letter sigma */
                map1.addElement(new MapElement((char)932,      "Tau"));         /* Greek capital letter tau */
                map1.addElement(new MapElement((char)933,      "Upsilon"));     /* Greek capital letter upsilon */
                map1.addElement(new MapElement((char)934,      "Phi"));         /* Greek capital letter phi */
                map1.addElement(new MapElement((char)935,      "Chi"));         /* Greek capital letter chi */
                map1.addElement(new MapElement((char)936,      "Psi"));         /* Greek capital letter psi */
                map1.addElement(new MapElement((char)937,      "Omega"));       /* Greek capital letter omega */
                map1.addElement(new MapElement((char)945,      "alpha"));       /* Greek small letter alpha */
                map1.addElement(new MapElement((char)946,      "beta"));        /* Greek small letter beta */
                map1.addElement(new MapElement((char)947,      "gamma"));       /* Greek small letter gamma */
                map1.addElement(new MapElement((char)948,      "delta"));       /* Greek small letter delta */
                map1.addElement(new MapElement((char)949,      "epsilon"));     /* Greek small letter epsilon */
                map1.addElement(new MapElement((char)950,      "zeta"));        /* Greek small letter zeta */
                map1.addElement(new MapElement((char)951,      "eta"));         /* Greek small letter eta */
                map1.addElement(new MapElement((char)952,      "theta"));       /* Greek small letter theta */
                map1.addElement(new MapElement((char)953,      "iota"));        /* Greek small letter iota */
                map1.addElement(new MapElement((char)954,      "kappa"));       /* Greek small letter kappa */
                map1.addElement(new MapElement((char)955,      "lambda"));      /* Greek small letter lambda */
                map1.addElement(new MapElement((char)956,      "mu"));          /* Greek small letter mu */
                map1.addElement(new MapElement((char)957,      "nu"));          /* Greek small letter nu */
                map1.addElement(new MapElement((char)958,      "xi"));          /* Greek small letter xi */
                map1.addElement(new MapElement((char)959,      "omicron"));     /* Greek small letter omicron */
                map1.addElement(new MapElement((char)960,      "pi"));          /* Greek small letter pi */
                map1.addElement(new MapElement((char)961,      "rho"));         /* Greek small letter rho */
                map1.addElement(new MapElement((char)962,      "sigmaf"));      /* Greek small letter final sigma */
                map1.addElement(new MapElement((char)963,      "sigma"));       /* Greek small letter sigma */
                map1.addElement(new MapElement((char)964,      "tau"));         /* Greek small letter tau */
                map1.addElement(new MapElement((char)965,      "upsilon"));     /* Greek small letter upsilon */
                map1.addElement(new MapElement((char)966,      "phi"));         /* Greek small letter phi */
                map1.addElement(new MapElement((char)967,      "chi"));         /* Greek small letter chi */
                map1.addElement(new MapElement((char)968,      "psi"));         /* Greek small letter psi */
                map1.addElement(new MapElement((char)969,      "omega"));       /* Greek small letter omega */
                map1.addElement(new MapElement((char)977,      "thetasym"));    /* Greek theta symbol */
                map1.addElement(new MapElement((char)978,      "upsih"));       /* Greek upsilon with hook symbol */
                map1.addElement(new MapElement((char)982,      "piv"));         /* Greek pi symbol */
                map1.addElement(new MapElement((char)8194,     "ensp"));        /* en space */
                map1.addElement(new MapElement((char)8195,     "emsp"));        /* em space */
                map1.addElement(new MapElement((char)8201,     "thinsp"));      /* thin space */
                map1.addElement(new MapElement((char)8204,     "zwnj"));        /* zero width non-joiner */
                map1.addElement(new MapElement((char)8205,     "zwj"));         /* zero width joiner */
                map1.addElement(new MapElement((char)8206,     "lrm"));         /* left-to-right mark */
                map1.addElement(new MapElement((char)8207,     "rlm"));         /* right-to-left mark */
                map1.addElement(new MapElement((char)8211,     "ndash"));       /* en dash */
                map1.addElement(new MapElement((char)8212,     "mdash"));       /* em dash */
                map1.addElement(new MapElement((char)8216,     "lsquo"));       /* left single quotation mark */
                map1.addElement(new MapElement((char)8217,     "rsquo"));       /* right single quotation mark */
                map1.addElement(new MapElement((char)8218,     "sbquo"));       /* single low-9 quotation mark */
                map1.addElement(new MapElement((char)8220,     "ldquo"));       /* left double quotation mark */
                map1.addElement(new MapElement((char)8221,     "rdquo"));       /* right double quotation mark */
                map1.addElement(new MapElement((char)8222,     "bdquo"));       /* double low-9 quotation mark */
                map1.addElement(new MapElement((char)8224,     "dagger"));      /* dagger */
                map1.addElement(new MapElement((char)8225,     "Dagger"));      /* double dagger */
                map1.addElement(new MapElement((char)8226,     "bull"));        /* bullet */
                map1.addElement(new MapElement((char)8230,     "hellip"));      /* horizontal ellipsis */
                map1.addElement(new MapElement((char)8240,     "permil"));      /* per mille sign */
                map1.addElement(new MapElement((char)8242,     "prime"));       /* prime */
                map1.addElement(new MapElement((char)8243,     "Prime"));       /* double prime */
                map1.addElement(new MapElement((char)8249,     "lsaquo"));      /* single left-pointing angle quotation mark */
                map1.addElement(new MapElement((char)8250,     "rsaquo"));      /* single right-pointing angle quotation mark */
                map1.addElement(new MapElement((char)8254,     "oline"));       /* overline */
                map1.addElement(new MapElement((char)8260,     "frasl"));       /* fraction slash */
                map1.addElement(new MapElement((char)8364,     "euro"));        /* euro sign */
                map1.addElement(new MapElement((char)8465,     "image"));       /* black-letter capital i */
                map1.addElement(new MapElement((char)8472,     "weierp"));      /* script capital pXCOMMAX Weierstrass p */
                map1.addElement(new MapElement((char)8476,     "real"));        /* black-letter capital r */
                map1.addElement(new MapElement((char)8482,     "trade"));       /* trademark sign */
                map1.addElement(new MapElement((char)8501,     "alefsym"));     /* alef symbol */
                map1.addElement(new MapElement((char)8592,     "larr"));        /* leftwards arrow */
                map1.addElement(new MapElement((char)8593,     "uarr"));        /* upwards arrow */
                map1.addElement(new MapElement((char)8594,     "rarr"));        /* rightwards arrow */
                map1.addElement(new MapElement((char)8595,     "darr"));        /* downwards arrow */
                map1.addElement(new MapElement((char)8596,     "harr"));        /* left right arrow */
                map1.addElement(new MapElement((char)8629,     "crarr"));       /* downwards arrow with corner leftwards */
                map1.addElement(new MapElement((char)8656,     "lArr"));        /* leftwards double arrow */
                map1.addElement(new MapElement((char)8657,     "uArr"));        /* upwards double arrow */
                map1.addElement(new MapElement((char)8658,     "rArr"));        /* rightwards double arrow */
                map1.addElement(new MapElement((char)8659,     "dArr"));        /* downwards double arrow */
                map1.addElement(new MapElement((char)8660,     "hArr"));        /* left right double arrow */
                map1.addElement(new MapElement((char)8704,     "forall"));      /* for all */
                map1.addElement(new MapElement((char)8706,     "part"));        /* partial differential */
                map1.addElement(new MapElement((char)8707,     "exist"));       /* there exists */
                map1.addElement(new MapElement((char)8709,     "empty"));       /* empty set */
                map1.addElement(new MapElement((char)8711,     "nabla"));       /* nabla */
                map1.addElement(new MapElement((char)8712,     "isin"));        /* element of */
                map1.addElement(new MapElement((char)8713,     "notin"));       /* not an element of */
                map1.addElement(new MapElement((char)8715,     "ni"));          /* contains as member */
                map1.addElement(new MapElement((char)8719,     "prod"));        /* n-ary product */
                map1.addElement(new MapElement((char)8721,     "sum"));         /* n-ary summation */
                map1.addElement(new MapElement((char)8722,     "minus"));       /* minus sign */
                map1.addElement(new MapElement((char)8727,     "lowast"));      /* asterisk operator */
                map1.addElement(new MapElement((char)8730,     "radic"));       /* square root */
                map1.addElement(new MapElement((char)8733,     "prop"));        /* proportional to */
                map1.addElement(new MapElement((char)8734,     "infin"));       /* infinity */
                map1.addElement(new MapElement((char)8736,     "ang"));         /* angle */
                map1.addElement(new MapElement((char)8743,     "and"));         /* logical and */
                map1.addElement(new MapElement((char)8744,     "or"));          /* logical or */
                map1.addElement(new MapElement((char)8745,     "cap"));         /* intersection */
                map1.addElement(new MapElement((char)8746,     "cup"));         /* union */
                map1.addElement(new MapElement((char)8747,     "int"));         /* integral */
                map1.addElement(new MapElement((char)8756,     "there4"));      /* therefore */
                map1.addElement(new MapElement((char)8764,     "sim"));         /* tilde operator */
                map1.addElement(new MapElement((char)8773,     "cong"));        /* congruent to */
                map1.addElement(new MapElement((char)8776,     "asymp"));       /* almost equal to */
                map1.addElement(new MapElement((char)8800,     "ne"));          /* not equal to */
                map1.addElement(new MapElement((char)8801,     "equiv"));       /* identical toXCOMMAX equivalent to */
                map1.addElement(new MapElement((char)8804,     "le"));          /* less-than or equal to */
                map1.addElement(new MapElement((char)8805,     "ge"));          /* greater-than or equal to */
                map1.addElement(new MapElement((char)8834,     "sub"));         /* subset of */
                map1.addElement(new MapElement((char)8835,     "sup"));         /* superset of */
                map1.addElement(new MapElement((char)8836,     "nsub"));        /* not a subset of */
                map1.addElement(new MapElement((char)8838,     "sube"));        /* subset of or equal to */
                map1.addElement(new MapElement((char)8839,     "supe"));        /* superset of or equal to */
                map1.addElement(new MapElement((char)8853,     "oplus"));       /* circled plus */
                map1.addElement(new MapElement((char)8855,     "otimes"));      /* circled times */
                map1.addElement(new MapElement((char)8869,     "perp"));        /* up tack */
                map1.addElement(new MapElement((char)8901,     "sdot"));        /* dot operator */
                map1.addElement(new MapElement((char)8968,     "lceil"));       /* left ceiling */
                map1.addElement(new MapElement((char)8969,     "rceil"));       /* right ceiling */
                map1.addElement(new MapElement((char)8970,     "lfloor"));      /* left floor */
                map1.addElement(new MapElement((char)8971,     "rfloor"));      /* right floor */
                map1.addElement(new MapElement((char)9001,     "lang"));        /* left-pointing angle bracket */
                map1.addElement(new MapElement((char)9002,     "rang"));        /* right-pointing angle bracket */
                map1.addElement(new MapElement((char)9674,     "loz"));         /* lozenge */
                map1.addElement(new MapElement((char)9824,     "spades"));      /* black spade suit */
                map1.addElement(new MapElement((char)9827,     "clubs"));       /* black club suit */
                map1.addElement(new MapElement((char)9829,     "hearts"));      /* black heart suit */
                map1.addElement(new MapElement((char)9830,     "diams"));       /* black diamond suit */

                return map1;
        }



        /**
         * Build a unmodifiable Trie from entitiy Name to Character
         * @return Unmodifiable trie.
         */
   /*     private static synchronized Trie<Character> mkEntityToCharacterTrie()
        {
                Trie<Character> trie = new HashTrie<Character>();

                for(Map.Entry<Character,String> entry : characterToEntityMap.entrySet())
                        trie.put(entry.getValue(),entry.getKey());
                return Trie.Util.unmodifiable(trie);
        }*/
}
